/******************************************************************************\

  Copyright 2005 The University of North Carolina at Chapel Hill.
  All Rights Reserved.

  Permission to use, copy, modify and distribute this software and its
  documentation for educational, research and non-profit purposes, without
  fee, and without a written agreement is hereby granted, provided that the
  above copyright notice and the following three paragraphs appear in all
  copies. Any use in a commercial organization requires a separate license.

  IN NO EVENT SHALL THE UNIVERSITY OF NORTH CAROLINA AT CHAPEL HILL BE LIABLE
  TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
  DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
  ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF NORTH CAROLINA HAVE BEEN
  ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


  Permission to use, copy, modify and distribute this software and its
  documentation for educational, research and non-profit purposes, without
  fee, and without a written agreement is hereby granted, provided that the
  above copyright notice and the following three paragraphs appear in all
  copies.

  THE UNIVERSITY OF NORTH CAROLINA SPECIFICALLY DISCLAIM ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN
  "AS IS" BASIS, AND THE UNIVERSITY OF NORTH CAROLINA HAS NO OBLIGATION TO
  PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.


   ---------------------------------
  |Please send all BUG REPORTS to:  |
  |                                 |
  |   sungeui@cs.unc.edu               |
  |                                 |
   ---------------------------------


  The authors may be contacted via:

  US Mail:         Sung-Eui Yoon or Dinesh Manocha
                       Department of Computer Science
                       Sitterson Hall, CB #3175
                       University of North Carolina
                       Chapel Hill, NC 27599-3175

\*****************************************************************************/

#include "stdio.h"
#include "LayoutMetrics.h"
#include "math.h"
#include "VArray.h"

using namespace OpenCCL;




CMetrics::CMetrics (void)
{
	m_bMetricType = true;
	InitOrdering ();

}



void CMetrics::InitOrdering (void)
{
	//printf ("Init\n");
	//srand (time (NULL));

	m_hpBestOrdering.clear ();
	m_NumAddedOrdering = 0;
	m_BestOrdering = 0;
}

// Add an ordering
// Return value: positive value --> return ordering.
//		 negative value --> error.
int CMetrics::AddOrdering (int NumEdgeLengths, int * EdgeLengths)
{
	int i;

	m_NumAddedOrdering++;

	// check if we need to set the best ordering with an initial ordering.
	if (m_hpBestOrdering.empty ()) {	
		InitOrdering (NumEdgeLengths, EdgeLengths);

		m_BestOrdering = m_NumAddedOrdering;
		return m_NumAddedOrdering;
	}


	// check if added one is better than current better one.
	// If so, consider it as the best one.
	CHyperPlane NewOne = m_hpBestOrdering;	// not to affect the best one
	
	for (i = 0;i < NumEdgeLengths;i++)
	{
		int EL = EdgeLengths [i];

		assert (EL >= 0);

		CHyperPlane::iterator Iter = NewOne.find (EL);
		if (Iter == NewOne.end ()) {
			CHyperPlane::value_type NewMap (EL, 1);		
			NewOne.insert (NewMap);	// add one edge length
		}
		else 
			Iter->second += 1;
	}


	float ExpPF = PerformCOMetric (NewOne);

	if (ExpPF < 0) {
		m_hpBestOrdering.clear ();
		InitOrdering (NumEdgeLengths, EdgeLengths);

		m_BestOrdering = m_NumAddedOrdering;

	}


	return m_NumAddedOrdering;
}

//set a best ordering with the initial one.
void CMetrics::InitOrdering (int NumEdgeLengths, int * EdgeLengths)
{
	int i;

	assert (m_hpBestOrdering.empty ());
	
	for (i = 0;i < NumEdgeLengths;i++)
	{
		int EL = EdgeLengths [i];

		assert (EL >= 0);

		CHyperPlane::iterator Iter = m_hpBestOrdering.find (EL);
		if (Iter == m_hpBestOrdering.end ()) {
			CHyperPlane::value_type NewMap (EL, -1);			
			m_hpBestOrdering.insert (NewMap);	// add one edge length
		}
		else 
			Iter->second += (-1);
	}
}


float CMetrics::PerformCOMetric (CHyperPlane & HyperPlane)
{
	// HyperPlane is already sorted
	float Sum = 0, tempSum = 0;
	int i, Dim = HyperPlane.size ();
	int CoeffSum = 0;

	//printf ("Dim = %d\n", Dim);
	// centroid is (1/n, ... n/n)
	CHyperPlane::iterator Iter = HyperPlane.begin ();
	for (i = 0;i < Dim;i++)
	{
		CoeffSum += Iter->second;
		Sum += (float (Iter->second) * log (float (Iter->first)));	// cache-oblivious metric
		
		Iter++;
	}

	assert (CoeffSum == 0);




	return Sum;
}


int CMetrics::GetBestOrdering (void)
{
	return m_BestOrdering;
}
